
/*
 * $Id: PolyClt.cc,v 1.10 1999/09/26 23:36:15 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "AsyncClt.h"
#include "SyncClt.h"
#include "OLog.h"
#include "LogComment.h"
#include "SharedOpts.h"
#include "CltOpts.h"
#include "CltConnMgr.h"
#include "PolyClt.h"
#include "WorkSetFreezer.h"

#include "PglSemx.h"
#include "RobotSym.h"

#include "gadgets.h"
#include "rndDistrs.h"
#include "globals.h"
#include "shutdown.h"


Agent *PolyClt::makeAgent(const AgentSym *c) {
	Assert(c);

	// filter out robots
	if (!c->isA("Robot"))
		return 0;
	const RobotSym &cfg = (const RobotSym&)c->cast("Robot");

	double rr;
	Client *client = 0;
	if (cfg.reqRate(rr)) {
		client = new AsyncClt();
	} else {
		client = new SyncClt();
	}

	Assert(client);
	const bool bind2iface = manyHostAddrs();
	client->configure(&cfg, bind2iface);

	return client;
}

void PolyClt::configure() {

	PolyApp::configure();

	// working set freezer
	if (PglSemx::TheWorkSetLen >= 0)
		(void)new WorkSetFreezer(PglSemx::TheWorkSetLen);
}

void PolyClt::getOpts(Array<OptGrp*> &opts) {
	PolyApp::getOpts(opts);
	opts.append(&TheCltOpts);
}

void PolyClt::reportCfg() {
	PolyApp::reportCfg();
}

bool PolyClt::manyHostAddrs() const {
	NetAddr a;

	for (int i = 1; i < theLocalCfgs.count(); ++i) {
		if (!theLocalCfgs[i]->isA("Robot"))
			continue;
		if (!a)
			a = theLocalCfgs[i]->host();
		if (theLocalCfgs[i]->host() != a)
			return true;
	}
	return false; // all addresses are the same
}



int main(int argc, char *argv[]) {
	PolyClt app;
	return app.run(argc, argv);
}
