
/*
 * $Id: ServerRep.cc,v 1.2 1999/09/17 23:54:18 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "ServerRep.h"
#include "ErrorMgr.h"
#include "polyErrors.h"

// hard coded; how can we estimate these?
static const int TheNewOidsPrefetchDepth = 256;
static const int TheNewOidsPerReqMax = 16;
static const int TheNewOidSPerReqFactor = 4;

ServerRep::ServerRep(const NetAddr &anAddr): theAddr(anAddr),
	theNewOids(TheNewOidsPrefetchDepth),
	theSecOid(0), theWorkSetCap(-1), thePendCnt(0) {
	
}

int ServerRep::oidShortage() {
	//cerr << here << thePendCnt << " + " << theNewOids.count() << " = " << shortage << endl;
	int shortage = theNewOids.capacity() - (thePendCnt + theNewOids.count());
	if (shortage > TheNewOidsPerReqMax)
		shortage /= TheNewOidSPerReqFactor;
	thePendCnt += shortage;
	return shortage;
}

int ServerRep::newOid() {
	Assert(haveNewOids());
	return theNewOids.dequeue();
}

void ServerRep::noteNewOids(int oid, int askCnt, int gotCnt) {
	Assert(!gotCnt || oid > 0);

	thePendCnt -= askCnt;
	if (thePendCnt < 0)
		thePendCnt = 0;

	while (gotCnt-- > 0) {
		if (theNewOids.full()) {
			ReportError(errTooManyNewOids);
			break;
		}
		theNewOids.enqueue(oid++);
	}
}

void ServerRep::syncSrvInfo(const UniqId &aSrvId, int aSecOid) {
	if (!theId)
		theId = aSrvId;
	else
	if (theId != aSrvId) {
		ReportError(errSrvChangedWid);
		theNewOids.reset();
		theId = aSrvId;
		theSecOid = 0;
		thePendCnt = 0;
	}

	if (aSecOid > theSecOid)
		theSecOid = aSecOid;
}

void ServerRep::freezeWorkSetCap() {
	theWorkSetCap = theSecOid;
}
