
/*
 * $Id: StatIntvl.cc,v 1.8 1999/09/07 17:46:27 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <math.h>
#include <iomanip.h>

#include "Connection.h"
#include "Xaction.h"
#include "PolyOLog.h"
#include "StatIntvl.h"

#include "globals.h"
#include "gadgets.h"


StatIntvl::StatIntvl(StatIntvlRec &aRec): theRec(aRec) {
	Assert(TheConnOpenChannel);

	theChannels
		<< TheConnOpenChannel << TheConnEstChannel << TheConnCloseChannel
		<< TheXactBegChannel << TheXactEndChannel << TheXactErrChannel
		;
}

StatIntvl::~StatIntvl() {
}

void StatIntvl::noteConnEvent(BcastChannel *ch, const Connection *conn) {
	if (ch == TheConnOpenChannel)
		++theRec.theOpenLvl;
	else 
	if (ch == TheConnEstChannel)
		++theRec.theEstbLvl;
	else {
		Assert(ch == TheConnCloseChannel);
		if (!conn->bad()) {
			theRec.theConnLifeTm.record((TheClock - conn->openTime()).msec());
			theRec.theConnUseCnt.record(conn->useCnt());
		}
		if (conn->ioCnt())
			--theRec.theEstbLvl;
		--theRec.theOpenLvl;
	}
}

void StatIntvl::noteXactEvent(BcastChannel *ch, const Xaction *x) {
	if (ch == TheXactBegChannel) {
		++theRec.theXactLvl;
	} else
	if (ch == TheXactEndChannel) {
		const OidRec &oid = x->oidRec();
		theRec.theIdealHR.record(x->repSize(), oid.repeat && oid.cachable);
		theRec.theRealHR.record(x->repSize(), oid.hit);
		theRec.theChbR.record(x->repSize(), oid.cachable);
		if (oid.hit)
			theRec.theHitRespTm.record(x->lifeTime().msec());
		else {
			theRec.theFill.record(x->repSize());
			theRec.theMissRespTm.record(x->lifeTime().msec());
		}
		--theRec.theXactLvl;
	} else {
		Assert(ch == TheXactErrChannel);
		theRec.theErrCnt++;
		if (x)
			--theRec.theXactLvl;
	}
}

void StatIntvl::report(ostream &os) const {
	os
		<< ' ' << setw(6) << TheProgress.xacts()
		<< ' ' << setw(6) << theRec.repRate()
		<< ' ' << setw(6) << (int)rint(theRec.respTime().mean())
		<< ' ' << setw(6) << theRec.theRealHR.dhp()
		<< ' ' << setw(3) << theRec.theErrCnt
		;
}
