
/*
 * $Id: Connection.h,v 1.24 1999/09/22 21:59:51 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_CONNECTION_CLASS_H
#define PG_CONNECTION_CLASS_H

#include "Queue.h"
#include "NetAddr.h"
#include "Socket.h"
#include "Select.h"
#include "IOBuf.h"
#include "ConnCloseStat.h"

class ConnMgr;
class PortMgr;

// bi-directional connection based on a socket with I/O buffers

class Connection: public QueueItem {
	public:
		typedef ConnCloseStat::CloseKind CloseKind;

		class HalfPipe {
			public:
				HalfPipe(Connection &aConn, IOBuf &aBuf, IODir aDir):
					theConn(aConn), theBuf(aBuf), theDir(aDir) { reset(); }

				void reset();
				void start(Select::User *u);
				void stop(Select::User *u);

			public:
				Connection &theConn;
				IOBuf &theBuf;
				IODir theDir;
				FileScanReserv theReserv;
				int theIOCnt;
				bool isReady;
		};
		friend class HalfPipe;

	public:
		Connection();
		~Connection() { if (theSock) close(); }

		void reset();

		Socket &sock() { return theSock; }
		int fd() const { return theSock.fd(); }
		const NetAddr &addr() const { return theAddr; }
		int rport() const;
		ConnMgr *mgr() const { return theMgr; }

		bool exhausted() const { return isAtEof && !theRdBuf.contSize(); }
		bool atEof() const { return isAtEof; }
		bool bad() const { return isBad; }
		CloseKind closeKind() const { return theCloseKind; }

		bool connect(const NetAddr &addr, const SockOpt &opt, PortMgr *aPortMgr);
		bool accept(Socket &s, const SockOpt &opt, bool &fatal);
		Size read();
		Size write();
		bool close();

		void mgr(ConnMgr *aMgr) { theMgr = aMgr; }
		
		void useLimit(int aLimit) { theUseLmt = aLimit; }
		bool reusable() const;
		int useCnt() const { return theUseCnt; }
		Time useStart() const { return theUseStart; }
		void use();

		Time openTime() const { return theOpenTime; }
		int ioCnt() const { return theRd.theIOCnt + theWr.theIOCnt; }

		void bad(bool be) { isBad = be; }
		void lastUse(bool be) { isLastUse = be; }
		void closeKind(CloseKind aKind) { theCloseKind = aKind; }

	protected:
		void setSockOpt(const SockOpt &opt);
		bool preIO(HalfPipe &ioPipe);

	public:
		RdBuf theRdBuf;
		WrBuf theWrBuf;

		HalfPipe theRd; // these refer to bufs; must go after them!
		HalfPipe theWr;

	protected:
		Socket theSock;
		NetAddr theAddr;
		mutable ConnMgr *theMgr;
		PortMgr *thePortMgr;    // used to pass address to bind call

		int theLocPort;
		mutable int theRemPort; // used as a cache for sock().rport()

		Time theOpenTime;
		Time theUseStart;       // start of the current/last "use"
		int theUseLmt;          // maximum number of uses allowed
		int theUseCnt;          // number of "uses" since open()
		CloseKind theCloseKind; // how the connection was closed

		bool isBad;
		bool isAtEof;
		bool isLastUse;         // this must be the last use of the conn
};

#endif
