
/*
 * $Id: ILog.h,v 1.8 1999/09/07 17:47:00 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_INPUT_LOG_CLASS_H
#define PG_INPUT_LOG_CLASS_H

#include <iostream.h>

#include "Time.h"
#include "Size.h"
#include "NetDouble.h"
#include "BigSize.h"
#include "String.h"
#include "Array.h"
#include "Progress.h"

// every log entry has this prefix
struct LogEntryPx {
	Size theSize;
	int theTag;

	LogEntryPx(): theTag(-1) {}
	operator void*() const { return theTag > 0 ? (void*)-1 : 0; }
};


// binary log, input interface
class ILog {
	protected:
		typedef void (Manip)(class ILog &);

	public:
		ILog();
		~ILog();

		void stream(const String &aFileName, istream *aStream);
		const String &fileName() const { return theFileName; }
		istream *stream() { return theStream; }
		operator void*() { return theStream && theStream->good() ? theStream : 0; }

		int geti() { int x; return(geti(x)); }
		int geti(int &x) { get(&x, sizeof(x)); return x = ntohl(x); }
		int geti(int *&xs, int &count); // returns count
		String &gets(String &s);

		LogEntryPx begEntry();
		void endEntry();
		bool eoe() { return theStream->tellg() >= theEntryEnd; }

		const Progress &progress() const { return theProgress; }

	protected:
		void getHeader();
		void get(void *buf, int len) { theStream->read((char*)buf, len); }

	protected:
		istream *theStream;
		String theFileName;

		streampos theEntryEnd; // end of the current entry (last+1)
		LogEntryPx theCurPx;   // prefix of the current entry
		Progress theProgress;  // current "local time", #errs, etc.
};


/* logging of common types */

inline
ILog &operator >>(ILog &il, int &x) {
	il.geti(x);
	return il;
}

inline
ILog &operator >>(ILog &il, double &x) {
	NetDouble nd;
	il >> nd.mnt >> nd.exp;
	x = nd;
	return il;
}

inline
ILog &operator >>(ILog &il, String &s) {
	il.gets(s);
	return il;
}

inline
ILog &operator >>(ILog &il, Time &t) {
	t.tv_sec = il.geti();
	t.tv_usec = il.geti();
	return il;
}

inline
ILog &operator >>(ILog &il, BigSize &bs) {
	return il >> bs.theAcc >> bs.theCnt;
}

inline
ILog &operator >>(ILog &il, Size &sz) {
	return il >> sz.theSize;
}

inline
ILog &operator >>(ILog &il, ILog::Manip m) {
	m(il);
	return il; 
}

template <class Item>
inline
ILog &operator >>(ILog &log, Array<Item> &a) {
	const int cnt = log.geti();
	if (a.count()) { // use existing entries
		Assert(cnt <= a.count());
		for (int i = 0; i < cnt; ++i)
			log >> a.item(i);
	} else {
		for (int i = 0; i < cnt; ++i) {
			Item item;
			log >> item;
			a << item;
		}
	}
	return log;
}

#endif
