
/*
 * $Id: NotifMsg.h,v 1.4 1999/09/07 17:47:07 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_NOTIFICATION_MESSAGE_H
#define PG_NOTIFICATION_MESSAGE_H

#include "NetDouble.h"

class NetAddr;

// messages used to send notifications to some [remote] monitor

class NotifMsg {
	public:
		enum Type { mtNone, mtStatus };

	public:
		NotifMsg(int anId = mtNone): theId(anId), theSize(sizeof(*this)) {};

	protected:
		void hton();
		void ntoh();

	public:
		int theId;
		int theSize;
};

class StatusNotifMsg: public NotifMsg {
	public:
		StatusNotifMsg(const String &aLabel = 0);

		void hton();
		void ntoh();

	public:
		char theLabel[16];     // a short 0-terminated description of an experiment
		Time theStartTime;
		Time theSndTime;       // when the message is sent
		Time theRespTime;
		NetDouble theReqRate;  // req / sec
		NetDouble theRepRate;  // rep / sec
		NetDouble theBwidth;   // network I/O bandwidth (r/w) [B/s]
		NetDouble theDHR;
		NetDouble theConnUse;  // req / connection
		NetDouble theErrRatio;

		int theXactTotCnt;     // total so far
		int theErrTotCnt;      // total so far
		int theSockInstCnt;    // at the moment
};

// status message forwarded by the first recepient
class StatusFwdMsg: public StatusNotifMsg {
	public:
		StatusFwdMsg();
		StatusFwdMsg(const StatusNotifMsg &m, Time aRcvTime, const NetAddr &aSndAddr);

		void hton();
		void ntoh();

	public:
		Time theRcvTime;       // when the message is received
		struct {               // original sender (set by first recepient)
			in_addr addr;
			int port;
		} theSndAddr;
		int theCopyCnt;        // number of copies forwarded (to various recepients)
};

#endif
