
/*
 * $Id: Server.h,v 1.29 1999/09/27 19:56:44 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_SERVER_CLASS_H
#define PG_SERVER_CLASS_H

#include "Socket.h"
#include "FileScanner.h"
#include "Farm.h"
#include "Agent.h"
#include "ContentCfg.h"
#include "SrvHistory.h"
#include "SrvXact.h"
#include "SrvConnMgr.h"

class RndDistr;
class OidRec;
class ServerSym;

class Server: public Agent, public FileScanner::User, public SrvConnMgr::User {
	public:
		Server();
		~Server();

		void configure(const ServerSym *cfg, bool bind2iface);
		void start();

		// xactions need access to this
		ContentCfg *selectContent(const OidRec &or);
		SrvHistory &history() { return theHistory; }

		void noteXactDone(SrvXact *x);

		virtual void noteReadReady(int fd);
		virtual void noteConnReady(Connection *conn);

	protected:
		void configureContents(const ServerSym *cfg);
		void startXact(Connection *conn);
		void deaf();

	protected:
		static ObjFarm<SrvXact> TheXacts;
		Array<ContentCfg*> theDirectAccess;  // directly accessible content
		RndDistr *theDirAccSel;              // selector for the above
		SrvConnMgr *theConnMgr;
		Socket theSock;

		SrvHistory theHistory;

		FileScanReserv theReserv;
};

#endif
