
/*
 * $Id: AgentSym.cc,v 1.7 1999/09/07 17:47:36 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "String.h"
#include "PglRec.h"
#include "PglDistrSym.h"
#include "PglNetAddrSym.h"
#include "PglStringSym.h"
#include "DumperSym.h"
#include "SocketSym.h"
#include "AgentSym.h"


AgentSym::AgentSym(const String &aType): RecSym(aType, new PglRec) {

	// add fields

	//theRec->bAdd("Dumper", "dump", new DumperSym);
	theRec->bAdd("Socket", "socket", new SocketSym);

	theRec->bAdd("string", "kind", 0);
	theRec->bAdd("time_distr", "xact_think", 0);
	theRec->bAdd("int_distr", "pconn_use_lmt", 0);

	theRec->bAdd("addr", "host", 0);
}

AgentSym::AgentSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
	// find fields
}

bool AgentSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "Agent";
}

SynSym *AgentSym::clone(const String &type) const {
	if (isA(type))
		return new AgentSym(this->type(), theRec->clone());
	return RecSym::clone(type);
}

String AgentSym::kind() const {
	SynSymTblItem *ki = 0;
	Assert(theRec);
	Assert(theRec->find("kind", ki));
	return ki->sym() ?
		((const StringSym&)ki->sym()->cast("string")).val() : String(0);
}

NetAddr AgentSym::host() const {
	SynSymTblItem *host = 0;
	Assert(theRec);
	Assert(theRec->find("host", host));
	return host->sym() ?
		((const NetAddrSym&)host->sym()->cast("addr")).val() : NetAddr();
}

SocketSym *AgentSym::socket() const {
	SynSymTblItem *si = 0;
	Assert(theRec->find("socket", si));
	Assert(si->sym());
	return &(SocketSym&)si->sym()->cast("Socket");
}

RndDistr *AgentSym::xactThink() const {
	return getDistr("xact_think");
}

RndDistr *AgentSym::pconnUseLmt() const {
	return getDistr("pconn_use_lmt");
}
