
/*
 * $Id: ObjLifeCycleSym.cc,v 1.3 1999/09/07 17:47:40 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "String.h"
#include "PglRec.h"
#include "PglArraySym.h"
#include "PglDistrSym.h"
#include "ObjLifeCycleSym.h"


ObjLifeCycleSym::ObjLifeCycleSym(): RecSym("ObjLifeCycle", new PglRec) {

	// add fields

	theRec->bAdd("time_distr", "birthday", 0);
	theRec->bAdd("time_distr", "length", 0);
	theRec->bAdd("float", "variance", 0);
	theRec->bAdd("float", "with_lmt", 0);
	theRec->bAdd("time_distr[]", "expires", 0);
}

ObjLifeCycleSym::ObjLifeCycleSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
	// find fields
}

bool ObjLifeCycleSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "ObjLifeCycle";
}

SynSym *ObjLifeCycleSym::clone(const String &type) const {
	if (isA(type))
		return new ObjLifeCycleSym(this->type(), theRec->clone());
	return RecSym::clone(type);
}

RndDistr *ObjLifeCycleSym::bday() const {
	return getDistr("birthday");
}

RndDistr *ObjLifeCycleSym::length() const {
	return getDistr("length");
}

bool ObjLifeCycleSym::variance(double &ratio) const {
	return getDouble("variance", ratio);
}

bool ObjLifeCycleSym::withLmt(double &ratio) const {
	return getDouble("with_lmt", ratio);
}

bool ObjLifeCycleSym::expires(Array<RndDistr*> &times, RndDistr *&selector, Array<QualifSym*> &qs) const {
	SynSymTblItem *ti = 0;
	Assert(theRec->find("expires", ti));
	if (!ti->sym())
		return false; // undefined

	ArraySym &a = (ArraySym&)ti->sym()->cast("Array");
	selector = a.makeSelector("expires");
	for (int i = 0; i < a.count(); ++i) {
		DistrSym &ds = (DistrSym&)a[i]->cast("Distr");
		times.append(ds.val());
		qs.append(ds.qualifier());
	}

	return true;
}
