
/*
 * $Id: PglListSym.cc,v 1.3 1999/09/07 17:47:44 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "PglListSym.h"


ListSym::ListSym(const String &aType): ExpressionSym(aType) {
}

ListSym::~ListSym() {
	while (count()) delete pop();
}

bool ListSym::isA(const String &type) const {
	return ExpressionSym::isA(type) || type == "List";
}

SynSym *ListSym::clone(const String &type) const {
	if (!isA(type))
		return 0;

	ListSym *clone = new ListSym("List");

	if (!clone->absorb(*this)) {
		delete clone;
		return 0;
	}

	return clone;
}

void ListSym::add(const SynSym &s) {
	if (!cadd(s))
		addFailure(s);
}

void ListSym::addFailure(const SynSym &) {
	Assert(0);
}

bool ListSym::cadd(const SynSym &s) {
	doAdd(s.dup(), s);
	return true;
}

void ListSym::doAdd(SynSym *sclone, const SynSym &s) {
	Assert(sclone);
	sclone->loc(s.loc());
	append(sclone);
}

bool ListSym::absorb(const ListSym &arr) {
	const int scount = count();

	for (int i = 0; i < arr.count(); ++i) {
		Assert(arr[i]);
		if (!cadd(*arr[i])) {
			// restore previous state
			while (count() > scount) delete pop();
			return false;
		}
	}

	return true;
}

ostream &ListSym::print(ostream &os, const String &pfx) const {
	os << '[';
	for (int i = 0; i < count(); ++i) {
		if (i) os << ',';
		item(i)->print(os, pfx+"    ");
	}
	return os << ']';
}
