
/*
 * $Id: PglQualifSym.cc,v 1.6 1999/09/07 17:47:46 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "Assert.h"
#include "String.h"
#include "Rnd.h"
#include "PglDistrSym.h"
#include "PglQualifSym.h"


QualifSym::QualifSym(Kind aKind): ExpressionSym("qualifier"), theKind(aKind) {
}

bool QualifSym::isA(const String &type) const {
	return ExpressionSym::isA(type) || type == "Qualif";
}

SynSym *QualifSym::clone(const String &type) const {
	if (isA(type))
		return new QualifSym(theKind);
	return ExpressionSym::clone(type);
}

// lmt + expr  or  now + expr
ExpressionSym *QualifSym::bnOper(const String &op, const SynSym &exp) const {
	if (op != "+")
		return ExpressionSym::bnOper(op, exp);

	Assert(theKind != qfNone);
	DistrSym *d = (DistrSym*)exp.clone("time_distr");
	if (!d)
		cerr << exp.loc() << "`time_distr' expected after a qualifier; got `" << exp.type() << "'" << endl << xexit;
	d->qualifier(this, op);
	return d;
}

ostream &QualifSym::print(ostream &os, const String &) const {
	if (theKind == qfLmt)
		os << "lmt";
	else
	if (theKind == qfNow)
		os << "now";
	return os;
}
