
/*
 * $Id: polyOpts.cc,v 1.13 1999/09/28 02:43:21 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "Select.h"
#include "Poll.h"
#include "rndDistrs.h"
#include "opts.h"
#include "polyOpts.h"
#include "polyVersion.h"
#include "gadgets.h"


/* FileScanOpt */

FileScanOpt::FileScanOpt(OptGrp *aGrp, const char *aName, const char *aDescr, FileScanner *def):
	Opt(aGrp, aName, aDescr), theVal(def) {
}

bool FileScanOpt::parse(const char *name, const char *val) {

	delete theVal; theVal = 0;

	const String v = val;
	if (v == "select")
		theVal = new Select();
	else
	if (v == "poll")
		theVal = new Poll();
	else
		cerr << "unknown file scanner name (use `poll' or `select') for the `" << name << "' option; got: " << v << endl;

	return theVal;
}

void FileScanOpt::report(ostream &os) const {
	os << (theVal ? theVal->name() : "<none>");
}


/* DumpFlagsOpt */

DumpFlagsOpt::DumpFlagsOpt(OptGrp *aGrp, const char *aName, const char *aDescr):
	ListOpt(aGrp, aName, aDescr) {
	theAllFlags = theTypeFlags = thePartFlags = 0;
}

bool DumpFlagsOpt::addItem(const String &item) {
	if (item == "req")
		theAllFlags |= dumpReq, theTypeFlags |= dumpReq;
	else
	if (item == "rep")
		theAllFlags |= dumpRep, theTypeFlags |= dumpRep;
	else
	if (item == "err")
		theAllFlags |= dumpErr, theTypeFlags |= dumpErr;
	else
	if (item == "hdr")
		theAllFlags |= dumpHdr, thePartFlags |= dumpHdr;
	else
	if (item == "body")
		theAllFlags |= dumpBody, thePartFlags |= dumpBody;
	else
	if (item == "All")
		theAllFlags = theTypeFlags = thePartFlags = ~0U;
	else {
		cerr << "unknown flag `" << item << "' for the `" << name() << "' option" << endl;
		return false;
	}
	return true;
}

bool DumpFlagsOpt::operator ()(unsigned int type, unsigned int part) const {
	if (!theAllFlags)                            // disabled
		return false;
	if (theTypeFlags && !(theTypeFlags & type))  // restricted to other
		return false;
	if (thePartFlags && !(thePartFlags & part))  // restricted to other
		return false;
	return true;
}

void DumpFlagsOpt::report(ostream &os) const {
	if (theAllFlags & dumpReq)
		os << "req,";
	if (theAllFlags & dumpRep)
		os << "rep,";
	if (theAllFlags & dumpErr)
		os << "err,";
	if (theAllFlags & dumpHdr)
		os << "hdr,";
	if (theAllFlags & dumpBody)
		os << "body,";
}


/* HostTypeOpt */

HostTypeOpt::HostTypeOpt(OptGrp *aGrp, const char *aName, const char *aDescr):
	Opt(aGrp, aName, aDescr) {
}

bool HostTypeOpt::parse(const char *, const char *) {
	Assert(theCmdLine);
	cerr
		<< "vesrion " << PolyVersion()
		<< " built on " << CONFIG_HOST_TYPE
		<< " at " << PolyCompTime() << ' ' << PolyCompDate()
		<< endl;
	exit(0);
}

void HostTypeOpt::report(ostream &os) const {
	os << CONFIG_HOST_TYPE;
}


/* VersionOpt */

VersionOpt::VersionOpt(OptGrp *aGrp, const char *aName, const char *aDescr):
	Opt(aGrp, aName, aDescr) {
}

bool VersionOpt::parse(const char *, const char *) {
	Assert(theCmdLine);
	cerr << PolyLetterhd() << endl;
	exit(0);
}

void VersionOpt::report(ostream &os) const {
	os << PolyVersion();
}
