
/*
 * $Id: GParser.h,v 1.4 1999/09/07 17:48:19 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef XPARSER_GENERATED_PARSER_H
#define XPARSER_GENERATED_PARSER_H

#include "Array.h"
#include "SynSym.h"
#include "Lexer.h"

class ReduceInfo;


// The GParser class is the base class from which generated parser classes
// are derived from.

// The GParser class provides the general bottom-up driver routine
// for LR(1) parsing.  It includes two stacks: the first is the
// push-down stack for parsing, while the second is for semantic
// attributes.

// It also continues hooks for 3 other tables: the actionTable
// merges parsing action and transition information.  A position
// value indicates a shift to a destination state.  A negative
// value indicates a reduction whose details are found in the
// reduceTable.  Conditionals are indicated by the condTable
// pointer.

class GParser {
	public:
		GParser(Lexer *);
		virtual ~GParser();

		int parse();

	protected:
		virtual int actionTable(int, int) = 0;
		virtual ReduceInfo *reduceTable(int) = 0;

		int top() { return theParseStack.last(); }
		void push(int x) { theParseStack.push(x); }
		void throwAway(int x) { theParseStack.pop(x); }

		SynSym **sem_top(int len) { return &theSemStack.last(len); }
		void sem_push(SynSym *x);
		void sem_throwAway(int x);
		void peek();
		const TokenLoc &tokenLoc() const { return lexer->token().loc(); }

	protected:
		Lexer *lexer;

		int maxState;
		int acceptState;
		int *condTable;

		Array<SynSym*> theSemStack;
		Array<int> theParseStack;
};


struct ReduceInfo {
	int head;
	int parseItems;
	int semItems;
	SynSym *(*action)(SynSym **);
	int   (*conditional)(SynSym **);
};

#endif
