
/*
 * $Id: LexToken.h,v 1.3 1999/09/07 17:48:20 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef XPARSER_LEX_TOKEN_H
#define XPARSER_LEX_TOKEN_H

#include "SrcLoc.h"

class ostream;

// Lex token used by parsers and toke components


// token location in the source stream
typedef SrcLoc TokenLoc;


struct LexTokenChar {
	TokenLoc loc;
	char ch;
	bool isFirstOnLine;

	LexTokenChar();
};

class LexToken {
	public:
		LexToken() { reset(); }
		LexToken(int anId, const TokenLoc &aLoc) { reset(); id(anId); loc(aLoc); }

		void reset();

		const TokenLoc &loc() const { return theLoc; }
		int id() const { return theId; }
		const String &spell() const { return theSpell; }
		bool firstOnLine() const { return isFirstOnLine; }
		
		LexToken &operator +=(const LexTokenChar &c);

		void id(int anId) { theId = anId; }
		void loc(const TokenLoc &aLoc) { theLoc = aLoc; }
		void firstOnLine(bool be) { isFirstOnLine = be; }

		void trim(char c);

	protected:
		void addChar(char c);

	protected:
		TokenLoc theLoc;
		int theId;
		String theSpell;
		bool isFirstOnLine;
};

#endif
