
/*
 * $Id: Clock.cc,v 1.2 1999/09/07 17:48:25 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "xstd.h"

#include "Array.h"
#include "Clock.h"

static void RegisterClock(Clock *clock);
static void UnregisterClock(Clock *clock);

Clock TheClock;
static Array<Clock*> *TheClocks = 0;

/* Clock */

Time Clock::TheStartTime;

Clock::Clock() {
	RegisterClock(this);
}

Clock::~Clock() {
	UnregisterClock(this);
}


Time Clock::Update(bool advanceAll) {
	const Time curTime = Time::Now();
	const Time delay = curTime - TheClock;

	if (TheStartTime < 0)
		TheStartTime = curTime;

	TheClock.update(curTime);
	if (advanceAll) {
		for (int i = TheClocks->count()-1; i >=0 ; --i) {
			Clock *&c = TheClocks->item(i);
			if (c && c != &TheClock)
				TheClocks->item(i)->update(curTime);
		}
	}
	return delay;
}


static
void RegisterClock(Clock *clock) {
	Assert(clock);
	TheClocks->append(clock);
}

static
void UnregisterClock(Clock *clock) {
	for (int i = TheClocks->count()-1; i >=0 ; --i) {
		if ((*TheClocks)[i] == clock) {
			(*TheClocks)[i] = (*TheClocks)[TheClocks->count()-1];
			TheClocks->pop(1);
			return;
		}
	}
	Assert(0); // clock not found
}

/* ClockLibCounter; internal stuff, ignore */

int ClockLibCounter::theUseCount = 0;

ClockLibCounter::ClockLibCounter() {
	if (!theUseCount++)
		TheClocks = new Array<Clock*>;
}

ClockLibCounter::~ClockLibCounter() {
	if (!--theUseCount) {
		Assert(TheClocks);
		Assert(!TheClocks->count());
		delete TheClocks;
		TheClocks = 0;
	}
}
