
/*
 * $Id: NetAddr.cc,v 1.6 1999/09/07 17:48:26 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "xstd.h"

#include <arpa/inet.h>
#include <iostream.h>

#include "NetAddr.h"

NetAddr NetAddr::MakeAddr(unsigned long net, unsigned long lna, int port) {
	return NetAddr(inet_makeaddr(net, lna), port);
}

unsigned long NetAddr::lna() const {
	return inet_lnaof(theAddrN);
}

unsigned long NetAddr::net() const {
	return inet_netof(theAddrN);
}

void NetAddr::syncA() const {
	theAddrA = theAddrN.s_addr == INADDR_NONE ?
		0 : inet_ntoa(theAddrN);
}

void NetAddr::syncN() const {
	if (theAddrA)
		theAddrN.s_addr = inet_addr(theAddrA);
	else
		theAddrN.s_addr = INADDR_NONE;
}

bool NetAddr::operator == (const NetAddr &addr) const {
	if (thePort != addr.thePort)
		return false;

	if (theAddrN.s_addr != INADDR_NONE && addr.theAddrN.s_addr != INADDR_NONE)
		return theAddrN.s_addr == addr.theAddrN.s_addr;

	return theAddrA == addr.theAddrA;
}

ostream &NetAddr::print(ostream &os) const {
	if (!theAddrA)
		syncA();

	if (theAddrA)
		os << theAddrA;
	else
	if (thePort >= 0)
		os << '*';
	else
		os << "<none>";

	if (thePort >= 0)
		os << ':' << thePort;

	return os;
}
